program readsaf91

!     Description:
!
!     Read 60-level diverse profile dataset from ECMWF.
!
!     Files are big-endian: compile this routine using, for example:
!       $ ifort     -convert big_endian            readsaf60.f90
!       $ gfortran  -fconvert=swap                 readsaf60.f90
!       $ pgfortran -byteswapio                    readsaf60.f90
!       $ nagfor    -convert=BIG_ENDIAN -kind=byte readsaf60.f90
!
!     History:
!     Version      Date        Comment
!     1            29/01/2020  James Hocking, based on readsaf91.f90
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".

implicit none

integer, parameter      :: klev = 60 ! number of vertical levels
integer, parameter      :: nslay = 4 ! number of sub-surface layers

 real(kind=8)       :: pap(klev)
 real(kind=8)       :: paph(klev+1)
 real(kind=8)       :: pres(klev)
 real(kind=4)       :: sistl(nslay), sstl(nslay), sswvl(nslay)
 real(kind=4)       :: stt(klev), swv(klev), so3(klev)
 real(kind=4)       :: sw(klev), scc(klev), sclw(klev), sciw(klev)
 real(kind=8)       :: spres
 real(kind=4)       :: scvl, scvh, stvl, stvh, sci
 real(kind=4)       :: sasn, srsn, stsn, ssd, ssr, sal, soro
 real(kind=4)       :: spsurf, sq2m, sst, srlsm, srlon, srlat, su10, sv10, st2m
 integer            :: jdat, jlev, i
character(len=1)    :: cvar
!

write(*,*) "Read the reduced (r) or full (f) profile dataset?"
read(*,*) cvar
write(*,*) cvar

if (trim(cvar) .ne. 'r'    .and. trim(cvar) .ne. 'f') then
  write(*,*) 'Input must be r (reduced) or f (full), start again'
  stop
endif

if (trim(cvar) == 'r') then
  open(1,file='60L_SDr.dat',form='unformatted')
else
  open(1,file='60L_SD.dat',form='unformatted')
endif

i = 0
do
  read(1,end=1) &
     jdat, &                         ! date (yyyymmddhh)
     srlon, &                        ! longitude (deg)
     srlat, &                        ! latitude (deg)
     srlsm, &                        ! land/see mask (1=land, 0=sea)
     sst, &                          ! surface skin temperature (K)
     spsurf, &                       ! surface pressure (hPa)
     su10, &                         ! 10-meter u wind (m/s)
     sv10, &                         ! 10-meter v wind (m/s)
     st2m, &                         ! 2-meter temperature (K)
     sq2m, &                         ! 2-meter specific humidity (kg/kg)
     (stt(jlev),jlev = 1,klev), &    ! temperature (K)
     (swv(jlev),jlev = 1,klev), &    ! specific humidity (kg/kg)
     (so3(jlev),jlev = 1,klev), &    ! specific ozone (kg/kg)
     (scc(jlev),jlev = 1,klev), &    ! cloud cover
     (sclw(jlev),jlev = 1,klev), &   ! cloud liquid water content (kg/kg)
     (sciw(jlev),jlev = 1,klev), &   ! cloud ice water content (kg/kg)
     (sw(jlev),jlev = 1,klev), &     ! vertical velocity (Pa/s)
     scvl, &                         ! low vegetation cover
     scvh, &                         ! high vegetation cover
     stvl, &                         ! type of low vegetation 
     stvh, &                         ! type of high vegetation 
     sci, &                          ! ice cover
     sasn, &                         ! snow albedo (0-1)
     srsn, &                         ! snow density (kg/m3)
     stsn, &                         ! snow temperature (K)
     ssd, &                          ! snow depth (m)
     ssr, &                          ! surface roughness (m)
     sal, &                          ! surface albedo (0-1)
     (sistl(jlev),jlev = 1,nslay), & ! layer ice temp. (K)  (top to bottom)
     (sstl(jlev),jlev = 1,nslay), &  ! layer soil temp. (K) (top to bottom)
     (sswvl(jlev),jlev = 1,nslay),&  ! layer soil volumetric water (m3/m3) (top to bottom)
     soro                            ! surface geometric height (m)

  ! compute full-level pressure (pap) and half-level pressures (paph) [Pa]
  ! note that all profile variables above are given at full levels
  spres = spsurf * 100.
  call ec_p60l(spres,pap,paph)
  pres(:)=pap(:) / 100.

  i = i + 1
enddo
1 write(*,*) 'Number of profiles found in the file:',i

end

 subroutine ec_p60l(spres,pap,paph)
        
!     Description:
!     Computes the 60-level vertical pressure grid
!       associated to the input surface pressure
!     All pressures are in Pa
        
 implicit none
 integer, parameter    :: nlev=60
 integer               :: jk
 real(kind=8)          :: spres
 real(kind=8)          :: aam(nlev+1), bbm(nlev+1)
 real(kind=8)          :: pap(nlev), paph(nlev+1)
        
 data aam / &
      0.000000,    20.000000,    38.425343, &
     63.647804,    95.636963,   134.483307, &
    180.584351,   234.779053,   298.495789, &
    373.971924,   464.618134,   575.651001, &
    713.218079,   883.660522,  1094.834717, &
   1356.474609,  1680.640259,  2082.273926, &
   2579.888672,  3196.421631,  3960.291504, &
   4906.708496,  6018.019531,  7306.631348, &
   8765.053711, 10376.126953, 12077.446289, &
  13775.325195, 15379.805664, 16819.474609, &
  18045.183594, 19027.695313, 19755.109375, &
  20222.205078, 20429.863281, 20384.480469, &
  20097.402344, 19584.330078, 18864.750000, &
  17961.357422, 16899.468750, 15706.447266, &
  14411.124023, 13043.218750, 11632.758789, &
  10209.500977,  8802.356445,  7438.803223, &
   6144.314941,  4941.778320,  3850.913330, &
   2887.696533,  2063.779785,  1385.912598, &
    855.361755,   467.333588,   210.393890, &
     65.889244,     7.367743,     0.000000, &
      0.000000 &
          /

 data bbm / &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000000000, 0.0000000000, 0.0000000000, &
  0.0000758235, 0.0004613950, 0.0018151561, &
  0.0050811190, 0.0111429105, 0.0206778757, &
  0.0341211632, 0.0516904071, 0.0735338330, &
  0.0996746942, 0.1300225109, 0.1643843204, &
  0.2024759352, 0.2439331412, 0.2883229554, &
  0.3351548910, 0.3838921487, 0.4339629412, &
  0.4847715795, 0.5357099175, 0.5861684084, &
  0.6355474591, 0.6832686067, 0.7287858129, &
  0.7715966105, 0.8112534285, 0.8473749161, &
  0.8796569109, 0.9078838825, 0.9319403172, &
  0.9518215060, 0.9676452279, 0.9796627164, &
  0.9882701039, 0.9940194488, 0.9976301193, &
  1.0000000000 &
          /
   
 do jk=1,nlev+1
   paph(jk)=aam(jk)+bbm(jk)*spres
 end do
 do jk=1,nlev
   pap(jk)=0.5*(paph(jk)+paph(jk+1))
 end do
   
 return
 end subroutine ec_p60l
